/*
 * Copyright (C) 1997  Warren Jasper
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/*
 * a2dc.h
 */

/* Default device major number */

/*#define ADC_BASE     0x300   */ /* Default base address */
/*#define ADC_IRQ           3  */ /* Default irq          */


#ifndef DEFAULT_MAJOR_DEV
#define DEFAULT_MAJOR_DEV   31   /* Default Major Device Number */
#endif
#define AD_CHANNELS          8   /* 8 Differential              */
#define DIO_PORTS            3   /* dio ports                   */

#define DEFAULT_FREQ      1000   /* Default frequency  1KHz     */
#define PORT_MIN         0x200   /* minimum base address used in autodetecting */
#define PORT_MAX         0x3e0   /* maximum base address allowed by board      */
#define PORT_STEP        0x10    /* base step in enhanced mode (32 bytes)      */
#define PORT1_SIZE       0x10    /* uses 16 bytes of registers                 */
#define PORT2_SIZE       0x8     /* uses  8 bytes of registers  82C55, 82C54   */

/* I/O Register locations */

/* NOTE: base_reg to be found dynamically at boot time. Not hard coded into driver */

#define DATA_AND_CHNLS   (base_reg)      /* A/D Data & Channel Register      */
#define MSB_DATA_BYTE    (base_reg+0x1)    
#define STATUS_REG       (base_reg+0x2)  /* Interrupt & Trigger Status       */
#define DIO_REG          (base_reg+0x3)  /* 4 Bit Digital I/O Register       */
#define CLR_INTERRUPT    (base_reg+0x4)  /* Write only, clears IRQDATA       */
#define CNTRL_REG        (base_reg+0x5)  /* Set interrupt and pacer modes    */
#define GAIN_Q_ADDR_REG  (base_reg+0x6)  /* Channel Gain Queue Address Reg   */
#define GAIN_Q_DATA_REG  (base_reg+0x7)  /* Channel Gain Queue Data Reg      */

#define COUNTERB_0_DATA  (base_reg+0x8)  /* Total Counter Upper              */
#define COUNTERB_1_DATA  (base_reg+0x9)  /* Total Counter Lower              */
#define COUNTERB_2_DATA  (base_reg+0xa ) /* Pretrigger Index                 */
#define COUNTERB_CONTROL (base_reg+0xb)  /* Total & Pretrig Counter Control  */

#define COUNTERA_0_DATA  (base_reg+0xc)  /* Counter 0 Data                   */
#define COUNTERA_1_DATA  (base_reg+0xd)  /* Pacer Clock Upper Counter 1      */
#define COUNTERA_2_DATA  (base_reg+0xe)  /* Pacer Clock Lower Counter 2      */
#define COUNTERA_CONTROL (base_reg+0xf)  /* Pacer Clock Control Register     */

/* 8255 Digital I/O  */
#define DIO_PORTA        (base_reg+0x400) /* Port A 8 bit I/O of 8255 */
#define DIO_PORTB        (base_reg+0x401) /* Port B 8 bit I/O of 8255 */
#define DIO_PORTC        (base_reg+0x402) /* Port C 4+4 bit of 8255 */
#define DIO_CNTRL_REG    (base_reg+0x403) /* Mode and Direction Control Register */

/* 82C54 Counter Counter */
#define COUNTER0         (base_reg+0x404) /* Counter 0 Load      */
#define COUNTER1         (base_reg+0x405) /* Counter 1 Load      */
#define COUNTER2         (base_reg+0x406) /* Counter 2 Load      */
#define COUNTER_CONTROL  (base_reg+0x407) /* Counter Control Reg */

/* Errors */

#define BAD_CHANNEL        1
#define BAD_COUNT          2
#define BAD_GAIN           3
#define NOT USED           4
#define BAD_SPEED          5
#define ADC_BUSY           6
#define INT_ALREADY_ACTIVE 7

/* General defines */

#define TRUE  1
#define FALSE 0

/* #define DEBUG */

/* DT Connection is not supported */

#define DT_CONNECT FALSE

/* Interrupt context data. Controls interrupt service handler */

typedef struct ADC_BOARD_REC {
  WORD base;           /* Base address of ADC board                  */
  WORD base2;          /* Base address + 0x400 of ADC board          */
  BYTE ir;             /* shifted bits for board 2,3,5,7,10,11,12,15 */
  int  irq;            /* IRQ inturrepts                             */
  LONG freq;           /* Complete frequency value                   */
  int  busy;           /* busy = TRUE, free = FALSE                  */
  WORD ctr1;           /* MSB of estimated "freq"                    */
  WORD ctr2;           /* LSB of estimated "freq"                    */
  BYTE statusReg;      /* value of control and status bits           */
  BYTE last_addr;      /* last written channel-gain address          */
  BYTE last_gain;
} ADC_BoardRec;

/* Values kept for each channel. */

typedef struct ADC_CHANNEL_REC {
  int  open;            /* Is channel device open()                 */
  int  adc_dio_pretrig; /* TRUE = write dio before sampling         */
  LONG count;           /* Number of samples requested              */
  WORD pretrigCount;    /* Number of samples before trigger         */
  BYTE chan;            /* Channel number                           */
  BYTE dio_value;       /* DIO value to be written before sampling  */
  BYTE mode;            /* 0 = Soft Trigger                         */
                        /* 2 = External Trigger                     */
                        /* 4 = Pacer Clock                          */
                        /* 6 = Pacer Clock External Trig Enable     */
  BYTE gain;            /* Voltage range for gain queue data reg    */
  int  update_queue;    /* TRUE = update gain queue on read()       */
} ADC_ChanRec;

typedef struct DIO_CHANNEL_REC {
  int  open;              /* Is channel device open()    */
  WORD value;             /* value of DIO Channel        */
  WORD addr;              /* address of the channel      */
  int mode;               /* mode for 8255               */
  unsigned short f_flags; /* flags sent by open()        */
} DIO_ChanRec;


/*************************************************************************
* STATUS_REG     base_reg+2                 Status Register              *
**************************************************************************/
#define IRQDATA    0x80    /* R/O Status of the IRQ flip flop                    */
#define TRGSTAT    0x40    /* R/O =1 if trigger has occurred when pretrig enable */
#define OVRUN      0x20    /* R/O  No overrun = 0, FIFO buffer full = 1          */
#define TOOFAST    0x10    /* R/W see documentation                              */
#define PRETRI     0x8     /* R/W Pretrigger Enabled = 1, Disable = 0            */
#define DTEN       0x4     /* R/W =1 DT-Connect Enable = 1, Disable = 0          */
#define CTR0       0x2     /* R/W selects source of CTR0 clock input             */
#define TRIG0      0x1     /* R/W extern trigger enabled = 1, disabled  = 0      */

/*************************************************************************
* DIO_REG     base_reg+3            4 Bit Digital I/O Register           *     
**************************************************************************/
#define DI3        0x8     /* Digital Input 3   */
#define DI2        0x4     /* Digital Input 2   */
#define DI1        0x2     /* Digital Input 1   */
#define DI0        0x1     /* Digital Input 0   */
#define DO3        0x8     /* Digital Output 3  */
#define DO2        0x4     /* Digital Output 2  */
#define DO1        0x2     /* Digital Output 1  */
#define DO0        0x1     /* Digital Output 0  */
#define CTR0_GATE  0x4     /* CTR0 Gate         */
#define TRIGIN     0x1     /* Trigger Input     */


/*************************************************************************
* CNTRL_REG    base_reg+5     Interrupt Control, Pacer Source Register   *
**************************************************************************/
#define INTEN   0x80   /* Interrupt enable = 1, Disable = 0       */
#define S1      0x2    /* S1=0 Software triggered A/D only        */
#define S0      0x1    /* S0=0 External Trigger, S0=1 Pacer Clock */


/*************************************************************************
* Constants for dealing with the 8254 counters                           *
**************************************************************************/

#define MODE0 0x0
#define MODE1 0x2
#define MODE2 0x4
#define MODE3 0x6
#define MODE4 0x8
#define MODE5 0xa

#define C0 0x00  /* Select Counter 0  */
#define C1 0x40  /* Select Counter 1  */
#define C2 0x80  /* Select Counter 2  */
#define RB 0xc0  /* Read-Back Command */

#define LATCH    0x00
#define LSBONLY  0x10
#define MSBONLY  0x20
#define LSBFIRST 0x30

#define PACKETSIZE  512
#define MAX_COUNT   16384   /* must not exceed 65528 */
